//
// Metallic Flakes Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//
// NOTE:
// The metallic surface consists of 2 layers - 
// 1. a polished layer of wax on top (contributes a smooth specular reflection and an environment mapped reflection with a Fresnel term), and 
// 2. a blue metallic layer with a sprinkling of gold metallic flakes underneath


// sparkle parameters
#define SPRINKLE    0.3
#define SCATTER     0.3

#define VOLUME_NOISE_SCALE  10

// scene info (EffectEdit specific controls)
string XFile = "tiny.x";
int    BCLR  = 0xff404080;

// textures

texture EnvironmentMap < string name = "Data\\Images\\Effects\\env3.bmp"; >;

// procedural texture that contains a normal map used for the metal sparkles
texture NoiseMap 
< 
    string type = "VOLUME"; 
    string function = "GenerateSparkle"; 
    int width = 32;
    int height = 32;
    int depth = 32; 
>;

// transformations
float4x3 WorldView  : WORLDVIEW;
float4x4 Projection : PROJECTION;

// light direction (view space)
float3 L < string UIDirectional = "Light Direction"; > = normalize(float3(-0.397f, -0.397f, 0.827f));

// light intensity
float4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
float4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
float4 I_s = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular

// material reflectivity
float4 k_a : MATERIALAMBIENT = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient  (metal)
float4 k_d : MATERIALDIFFUSE = { 0.1f, 0.1f, 0.9f, 1.0f };    // diffuse  (metal)
float4 k_s = { 0.4f, 0.3f, 0.1f, 1.0f };    // specular (metal)
float4 k_r = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular (wax)

struct VS_OUTPUT
{
    float4 Position   : POSITION;
    float4 Diffuse    : COLOR0;
    float4 Specular   : COLOR1;               
    float2 TexCoord   : TEXCOORD0;
//    float3 Reflection : TEXCOORD0;               
//    float3 NoiseCoord : TEXCOORD1;               
//    float3 Glossiness : TEXCOORD2;               
//    float3 HalfVector : TEXCOORD3;
};


// samplers
sampler Environment = sampler_state
{ 
    Texture = (EnvironmentMap);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

sampler SparkleNoise = sampler_state
{ 
    Texture = (NoiseMap);
    MipFilter = LINEAR; 
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

// vertex shader
VS_OUTPUT VS(    
    float3 Position : POSITION,
    float3 Normal   : NORMAL, 
    float3 Tangent  : TANGENT,
    float3 Color    : COLOR,
    float3 Binormal : BINORMAL,
    float2 TexCoord : TEXCOORD0)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    L = -L;

    float3 P = mul(float4(Position, 1), (float4x3)WorldView);   // position (view space)
    float3 N = normalize(mul(Normal, (float3x3)WorldView));     // normal (view space)
    float3 T = normalize(mul(Tangent, (float3x3)WorldView));    // tangent (view space)
    float3 B = cross(N, T);                                     // binormal (view space)
    float3 R = normalize(2 * dot(N, L) * N - L);                // reflection vector (view space)
    float3 V = -normalize(P);                                   // view direction (view space)
    float3 G = normalize(2 * dot(N, V) * N - V);                // glance vector (view space)
    float3 H = normalize(L + V);                                // half vector (view space)
    float  f = 0.5 - dot(V, N); f = 1 - 4 * f * f;              // fresnel term

    P += N*f*f;


    // position (projected)
    Out.Position = mul(float4(P, 1), Projection);             

    // diffuse + ambient (metal)
    Out.Diffuse = I_a * k_a + I_d * k_d * max(0, dot(N, L)); 

    // specular (wax)
    Out.Specular  = saturate(dot(H, N));
    Out.Specular *= Out.Specular;
    Out.Specular *= Out.Specular;
    Out.Specular *= Out.Specular;
    Out.Specular *= Out.Specular;                              
    Out.Specular *= Out.Specular;                        
    Out.Specular *= k_r;                                       

     // glossiness (wax)
//    Out.Glossiness = f * k_r;
    float4 Glossiness = f * k_r;


    // transform half vector into vertex space
    float3 HalfVector = float3(dot(H, N), dot(H, B), dot(H, T));   
    HalfVector = (1 + HalfVector) / 2;  // bias

    // environment cube map coordinates
    float3 Reflection = float3(-G.x, G.y, -G.z);                   

    // volume noise coordinates
//    Out.NoiseCoord = Position * VOLUME_NOISE_SCALE;             


//    Out.TexCoord = float2(HalfVector.x,HalfVector.y);

//    Out.TexCoord = float2(N.x,N.y);

    Out.TexCoord = float2(N.x,N.y) + float2(sin(dot(G, Normal)),sin(f));



//    Out.Diffuse *= GenerateSparkle(Position);

//    Out.Diffuse *= dot(Normal,Tangent);
    Out.Diffuse += Glossiness/2;
    Out.Diffuse += Out.Specular;
//Out.Diffuse = 1;

//Out.Diffuse /= 2;


//   float2 Gloss = texCUBE(Environment, Reflection) * saturate(Glossiness);              

//Out.TexCoord.x = Gloss.x;

//Out.Diffuse = Out.Position;

//Out.Position += float4(Normal.x,Normal.y,1,Normal.z)/60;



    return Out;


}

// technique
technique TMetallicFlakes
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
//        PixelShader  = compile ps_1_1 PS();
        PixelShader = NULL;

        AlphaBlendEnable = TRUE;
        CullMode         = CCW;
        DitherEnable = TRUE;

        // texture
        Texture[0] = (EnvironmentMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        AlphaOp[0]   = SELECTARG1;
        AlphaArg1[0] = DIFFUSE;
        AlphaArg2[0] = CURRENT;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = TRUE;
        SrcBlend         = SRCALPHA;
        DestBlend        = INVSRCALPHA;
//        BlendOp = MIN;


    }
}

