//
// Metallic Flakes Shader
// Copyright (c) Microsoft Corporation. All rights reserved.
//
// Note: This effect file works with EffectEdit.
//
// NOTE:
// The metallic surface consists of 2 layers - 
// 1. a polished layer of wax on top (contributes a smooth specular reflection and an environment mapped reflection with a Fresnel term), and 
// 2. a blue metallic layer with a sprinkling of gold metallic flakes underneath


// sparkle parameters
#define SPRINKLE    0.3
#define SCATTER     0.3

#define VOLUME_NOISE_SCALE  10

// scene info (EffectEdit specific controls)
string XFile = "tiny.x";
int    BCLR  = 0xff404080;

// textures

texture EnvironmentMap < string name = "Data\\Images\\Effects\\env3.bmp"; >;
texture TinyMap < string name = "Data\\Images\\Effects\\env3.bmp" ; >;


// transformations
half4x3 WorldView  : WORLDVIEW;
half4x4 Projection : PROJECTION;

// light direction (view space)
half3 L < string UIDirectional = "Light Direction"; > = normalize(half3(-0.397f, -0.397f, 0.827f));

// light intensity
half4 I_a = { 0.3f, 0.3f, 0.3f, 1.0f };    // ambient
half4 I_d = { 1.0f, 1.0f, 1.0f, 1.0f };    // diffuse
half4 I_s = { 0.7f, 0.7f, 0.7f, 1.0f };    // specular

// material reflectivity
half4 k_a : MATERIALAMBIENT = { 0.2f, 0.2f, 0.2f, 1.0f };    // ambient  (metal)
half4 k_d : MATERIALDIFFUSE = { 0.1f, 0.1f, 0.9f, 1.0f };    // diffuse  (metal)
half4 k_s = { 0.4f, 0.3f, 0.1f, 1.0f };    // specular (metal)
half4 k_r = { 0.5f, 0.5f, 0.5f, 0.5f };    // specular (wax)

half time : TIME;

struct VS_OUTPUT
{
    half4 Position   : POSITION;
    half4 Diffuse    : COLOR0;
    half4 Specular   : COLOR1;               
    half2 TexCoord   : TEXCOORD0;
//    half3 Reflection : TEXCOORD0;               
//    half3 NoiseCoord : TEXCOORD1;               
//    half3 Glossiness : TEXCOORD2;               
//    half3 HalfVector : TEXCOORD3;
};


// samplers
sampler Environment = sampler_state
{ 
    Texture = (EnvironmentMap);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


// vertex shader
VS_OUTPUT VS(    
    half3 Position : POSITION,
    half3 Normal   : NORMAL, 
    half3 Tangent  : TANGENT,
    half3 Color    : COLOR,
    half3 Binormal : BINORMAL)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    L = -L;

    half3 P = mul(half4(Position, 1), (half4x3)WorldView);   // position (view space)
    half3 N = normalize(mul(Normal, (half3x3)WorldView));     // normal (view space)
    half3 T = normalize(mul(Tangent, (half3x3)WorldView));    // tangent (view space)
    half3 B = cross(N, T);                                     // binormal (view space)
    half3 R = normalize(2 * dot(N, L) * N - L);                // reflection vector (view space)
    half3 V = -normalize(P);                                   // view direction (view space)
    half3 G = normalize(2 * dot(N, V) * N - V);                // glance vector (view space)
    half3 H = normalize(L + V);                                // half vector (view space)
    half  f = 0.5 - dot(V, N); f = 1 - 4 * f * f;              // fresnel term

    // animate the wisps:
    half Ny = max(0,N.y);
    N.y += Ny * .5*sin(time*3 * sin(Position.x)); 
    N.x += Ny * .5*cos(time*3 * cos(Position.y)); 
    N = saturate(N+0.5)-0.5; // keeps the wisps under control and somehow lets it run faster

    P += N*f*f*1.6;

    // position (projected)
    Out.Position = mul(half4(P, 1), Projection);             

    // diffuse + ambient (metal)
    Out.Diffuse = I_a * k_a + I_d * k_d * max(0, dot(N, L)); 

    // specular (wax)
    Out.Specular  = saturate(dot(H, N));
    Out.Specular = (half)pow(Out.Specular, 32);                     
    Out.Specular *= k_r;                                       

     // glossiness (wax)
    half4 Glossiness = f * k_r;

    // transform half vector into vertex space
    half3 HalfVector = half3(dot(H, N), dot(H, B), dot(H, T));   
    HalfVector = (1 + HalfVector) / 2;  // bias

    // environment cube map coordinates
    half3 Reflection = half3(-G.x, G.y, -G.z);                   

    Out.TexCoord = half2(N.x,N.y) + half2(sin(dot(G, Normal)),sin(f));

    Out.Diffuse *= Glossiness/2;
    Out.Diffuse += Out.Specular;

    Out.Diffuse += half4(N.y/4,0,N.y/2,0);

    return Out;
}

// technique
technique TPhantom
{
   pass P0
    {
        VertexShader = compile vs_1_1 VS();
//        PixelShader  = compile ps_1_1 PS();
        PixelShader = NULL;

//        AlphaBlendEnable = TRUE;
        CullMode         = CCW;
        ZEnable         = FALSE;
        DitherEnable = TRUE;

        // texture
//        Texture[0] = (EnvironmentMap);
        Texture[0] = (TinyMap);

        // sampler states
        MinFilter[0] = LINEAR;
        MagFilter[0] = LINEAR;
        MipFilter[0] = LINEAR;

        // set up texture stage states for single texture modulated by diffuse 
        ColorOp[0]   = MODULATE;
        ColorArg1[0] = TEXTURE;
        ColorArg2[0] = CURRENT;
        AlphaOp[0]   = SELECTARG1;
        AlphaArg1[0] = DIFFUSE;
        AlphaArg2[0] = CURRENT;

        ColorOp[1]   = DISABLE;
        AlphaOp[1]   = DISABLE;

       AlphaBlendEnable = TRUE;
        SrcBlend         = ONE;
        DestBlend        = INVSRCALPHA;
//        BlendOp = SUBTRACT;


    }
}

